*+
*+
*+    Source Module => DBF2WK1.PRG
*+
*+    Functions: Function _123()
*+               Function lopen()
*+               Function lput()
*+               Function lclose()
*+               Function lotwrd()
*+               Function getdecs()
*+
*+       Tables: dbusearea( .T.,, cInput, "DBF_FILE", .T. )
*+               use
*+
*+    Reformatted by Click! 1.11 on Sep-29-1997 at 11:38 pm
*+
*+

/*
Program Name : DBF2WK1.PRG
Author       : Andi Jahja
WWW          : http://www.geocities.com/~andijahja/index.html
Email        : andijahj@link.net.id
Purpose      : Translate dBase III File into Lotus 1-2-3 Format
Exceptions   : Does not process any Memo Fields or empty DBF files
Description  : Public Domain
Compile      : Clipper DBF2WKS /N /M /W /A /P
Linking      : RTLINK FI DBF2WKS, CFLOAT
Syntax       : _123(dbfFileName, LotusOutPutFileName)

Modified     : Ross McKenzie (ross@sencon.com) Melbourne Australia
24 Sept '97

- corrected date problem when using SET EPOCH
- now tests for empty DBF
- reports records processed and total to process
- permits use of colour scheme of main program
- minor speed improvements
- minor additions to documentation
- restores current dbf (if any) and its order
- restores cursor shape and status
- uses functions from the now public domain SuperLib
Msg(), Messyn(), MakeBox(), UnBox(), Sls_PopCol()

*/

#include "INKEY.CH"

*+
*+
*+    Function _123()
*+
*+
*+
function _123( cInput, cOutput )

local lOldArea      //is another DBF open
local nOldArea      //old dbf area
local nOldOrder     //old index order
local nOldCursor    //old cursor status
local nRec          //number of DBF records
local nCount     := 0                   //records progressed
local aStruct       //structure of the input DBF
local nStruct       //speedup for-loop limit
local x             //for-loop index
local nFlds         //qty of fields
local wkfile        //Lotus-format output file
local _123Handle    //filehandle
local CProgBox      //progress box

/*
  save the enviroment conditions
*/
lOldArea := used()  //is another DBF open?
IF lOldArea         // if Yes,
   nOldOrder := indexord()              //save the current order and
   nOldarea  := select()                //save the current area
endif

nOldCursor := setcursor( 0 )            //cursor OFF & save the old

/*
  check the input file
*/
cInput := alltrim( cInput )
if !file( cInput )
   msg( 5, "The input file " + cInput + " is missing. Cannot proceed..." )
   // tidy up on exit
   IF lOldArea
      select( nOldArea )
      set Order to ( nOldOrder )
   endif
   setcursor( nOldCursor )
   return .T.
endif

/*
  check or build the Lotus-Format filename
*/
WkFile := alltrim( cOutput )
If at( ".", cOutPut ) == 0
   wkfile := cOutput + ".WK1"
endif

/*
  are you allowed to "clobber" an existing one?
*/
if file( wkfile )
   IF !messyn( " The file " + wkfile + " already exists. Overwrite it?", ;
               "Yes, Overwrite it", "No, abort instead" )
      IF lOldArea
         select( nOldArea )
         set Order to ( nOldOrder )
      endif
      setcursor( nOldCursor )
      return .T.
   endif
endif

/*
  OK, now open the input DBF file and check it has records
*/
dbusearea( .T.,, cInput, "DBF_FILE", .T. )
nRec := Dbf_File->( lastrec() )         //how many records in dbf?
IF nRec == 0
   // empty dbf, so exit politely
   msg( 0, "There are no records in " + cInput + " to process. Cannot proceed..." )
   use
   IF lOldArea
      select( nOldArea )
      set Order to ( nOldOrder )
   endif
   setcursor( nOldCursor )
   return ""
endif

/*
  OK to proceed. Examine the input file
*/
aStruct    := Dbf_File->( dbstruct() )  //obtain the dbf structure
nFlds      := Dbf_File->( fcount() )    //speed improvement
_123Handle := lopen( wkfile )           //open the Lotus-format file
nStruct    := len( aStruct )            //speed improvement

for x := 1 to nStruct
   do case
   case aStruct[ x ] [ 2 ] == "D"
      fwrite( _123Handle, chr( 8 ) + chr( 0 ) + chr( 3 ) + chr( 0 ) + lotwrd( x - 1 ) + chr( 9 ) )

   case aStruct[ x ] [ 2 ] == "L"
      fwrite( _123Handle, chr( 8 ) + chr( 0 ) + chr( 3 ) + chr( 0 ) + lotwrd( x - 1 ) + chr( 2 ) )

   case aStruct[ x ] [ 2 ] == "C"
      fwrite( _123Handle, chr( 8 ) + chr( 0 ) + chr( 3 ) + chr( 0 ) + lotwrd( x - 1 ) + chr( aStruct[ x ] [ 3 ] ) )

   case aStruct[ x ] [ 2 ] == "N"
      if aStruct[ x ] [ 4 ] == 0
         fwrite( _123Handle, chr( 8 ) + chr( 0 ) + chr( 3 ) + chr( 0 ) + lotwrd( x - 1 ) + chr( int( ( aStruct[ x ] [ 3 ] + 2 ) * 4 / 3 ) ) )
      else
         fwrite( _123Handle, chr( 8 ) + chr( 0 ) + chr( 3 ) + chr( 0 ) + lotwrd( x - 1 ) + chr( aStruct[ x ] [ 3 ] ) )
      endif

   endcase

   lput( _123Handle, aStruct[ x ] [ 1 ], x - 1, 0 )
next

cProgBox := makeBox( 11, 10, 13, 70, sls_popcol() )         //better integration
setpos( 11, 12 )

dispout( " Conversion Progress ( " + alltrim( str( nRec ) ) + " records ) ", ;
         sls_popcol() )                 //tell'em more

setpos( 13, 12 )
dispout( " Esc to abort " )             //tell'em how to cancel

while !Dbf_File->( eof() ) .and. inkey() <> K_ESC
   setpos( 12, 11 )
   *		DispOut( Repl( Chr(219), (++nCount/nRec)*59), "W/N" )
   dispout( repl( chr( 219 ), ( ++ nCount / nRec ) * 59 ), selected() )

   for x := 1 to nFlds                  //speed improvement
      lput( _123Handle, Dbf_File->( fieldget( x ) ), x - 1, Dbf_File->( recno() ) )
   next

   Dbf_File->( dbskip( 1 ) )

enddo
unbox( cProgBox )   //remove the progress box

lclose( _123Handle )

/*
  provide some information about the completion status
*/
IF eof()
   msg( 0, "Conversion Complete ( " + alltrim( str( nRec ) ) + ;
        " records processed )" )
else
   msg( 0, "Incomplete Conversion ( only " + alltrim( str( nCount ) ) + ;
        " of " + alltrim( str( nRec ) ) + " records were processed )" )
endif

/*
  close the input database
*/
use

/*
  restore the original settings
*/
IF lOldArea
   select( nOldArea )
   set Order to ( nOldOrder )
endif
setcursor( nOldCursor )

return nil

*+
*+
*+    Function lopen()
*+
*+    Called from ( dbf2wk1.prg  )   1 - function _123()
*+
*+
*+
function lopen( cInput )

local handle := fcreate( cInput, 0 )

If handle <> - 1
   fwrite( handle, chr( 0 ) + chr( 0 ) + chr( 2 ) + chr( 0 ) + chr( 6 ) + chr( 4 ) )
endif

return handle

*+
*+
*+    Function lput()
*+
*+    Called from ( dbf2wk1.prg  )   2 - function _123()
*+
*+
*+
function lput( handle, val, col, row )

local mstr
local decs

do case
case valtype( val ) == "C"
   mstr := chr( 15 ) + chr( 0 ) + lotwrd( len( val ) + 7 ) + chr( 255 ) + lotwrd( col ) + lotwrd( row ) + "'" + val + chr( 0 )

case valtype( val ) == "D"
   mstr := chr( 14 ) + chr( 0 ) + chr( 13 ) + chr( 0 ) + chr( 249 ) + lotwrd( col ) + lotwrd( row ) + cfloat( val - ctod( '01/01/1900' ) + 2 )

case valtype( val ) == "N"
   decs := getdecs( val )
   if decs <> 0
      mstr := chr( 14 ) + chr( 0 ) + chr( 13 ) + chr( 0 ) + chr( 128 + decs ) + lotwrd( col ) + lotwrd( row ) + cfloat( val )
   else
      mstr := chr( 14 ) + chr( 0 ) + chr( 13 ) + chr( 0 ) + chr( 192 ) + lotwrd( col ) + lotwrd( row ) + cfloat( val )
   endif

case valtype( val ) == "L"
   mstr := chr( 15 ) + chr( 0 ) + chr( 8 ) + chr( 0 ) + chr( 255 ) + lotwrd( col ) + lotwrd( row ) + "'" + if( val, "Y", "N" ) + chr( 0 )

endcase

return fwrite( handle, mstr )

*+
*+
*+    Function lclose()
*+
*+    Called from ( dbf2wk1.prg  )   1 - function _123()
*+
*+
*+
function lclose( handle )

fwrite( handle, chr( 01 ) + chr( 0 ) + chr( 0 ) + chr( 0 ) )
fclose( handle )

return .t.

*+
*+
*+    Function lotwrd()
*+
*+    Called from ( dbf2wk1.prg  )   5 - function _123()
*+                                  11 - function lput()
*+
*+
*+
function lotwrd( mvalue )

return chr( mvalue % 256 ) + chr( int( mvalue / 256 ) )

*+
*+
*+    Function getdecs()
*+
*+    Called from ( dbf2wk1.prg  )   1 - function lput()
*+
*+
*+
function getdecs( mnum )

local smnum := str( mnum )
local at    := at( '.', smnum )

if at == 0
   return 0
endif

return len( smnum ) - at

*+ EOF: DBF2WK1.PRG
